#ifdef GP2X

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include "SDL.h"
#include "conf.h"

#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/soundcard.h>

#include "gp2x.h"
#include "menu.h"

#define SYS_CLK_FREQ 7372800
// system registers
static struct
{
        unsigned short SYSCLKENREG,SYSCSETREG,FPLLVSETREG,DUALINT920,DUALINT940,DUALCTRL940,DISPCSETREG;
}
system_reg;

//volatile unsigned short *MEM_REG;
unsigned MDIV,PDIV,SCALE;
volatile unsigned *arm940code;
//static int fclk;
static int cpufreq;

/*
  void cpuctrl_init()
  {
  MEM_REG=&gp2x_memregs[0];
  }*/

void set_FCLK(unsigned MHZ)
{
        unsigned v;
        unsigned mdiv,pdiv=3,scale=0;
        MHZ*=1000000;
        mdiv=(MHZ*pdiv)/SYS_CLK_FREQ;
        //printf ("Old value = %04X\r",MEM_REG[0x924>>1]," ");
        //printf ("APLL = %04X\r",MEM_REG[0x91A>>1]," ");
        mdiv=((mdiv-8)<<8) & 0xff00;
        pdiv=((pdiv-2)<<2) & 0xfc;
        scale&=3;
        v=mdiv | pdiv | scale;
        gp2x_memregs[0x910>>1]=v;
}

unsigned get_FCLK()
{
        return gp2x_memregs[0x910>>1];
}

unsigned get_freq_920_CLK()
{
        unsigned i;
        unsigned reg,mdiv,pdiv,scale;
        reg=gp2x_memregs[0x912>>1];
        mdiv = ((reg & 0xff00) >> 8) + 8;
        pdiv = ((reg & 0xfc) >> 2) + 2;
        scale = reg & 3;
        MDIV=mdiv;
        PDIV=pdiv;
        SCALE=scale;
        i = (gp2x_memregs[0x91c>>1] & 7)+1;
        return ((SYS_CLK_FREQ * mdiv)/(pdiv << scale))/i;
}
unsigned short get_920_Div()
{
        return (gp2x_memregs[0x91c>>1] & 0x7);
}

void gp2x_video_RGB_setscaling(int W, int H)
{
 float escalaw,escalah;
 int bpp=(gp2x_memregs[0x28DA>>1]>>9)&0x3;

 if(gp2x_memregs[0x2800>>1]&0x100) //TV-Out
 {
   escalaw=489.0; //RGB Horiz TV (PAL, NTSC)
   if (gp2x_memregs[0x2818>>1]  == 287) //PAL
     escalah=274.0; //RGB Vert TV PAL
   else if (gp2x_memregs[0x2818>>1]  == 239) //NTSC
     escalah=331.0; //RGB Vert TV NTSC
 }
 else //LCD
 {
   escalaw=1024.0; //RGB Horiz LCD
   escalah=320.0; //RGB Vert LCD
 }

 // scale horizontal
 gp2x_memregs[0x2906>>1]=(unsigned short)((float)escalaw *(W/320.0));
 // scale vertical
 gp2x_memregl[0x2908>>2]=(unsigned long)((float)escalah *bpp *(H/240.0));

}

//volatile Uint32 *gp2x_ram;
void gp2x_ram_init(void) {
	if(!gp2x_dev_mem) gp2x_dev_mem = open("/dev/mem",   O_RDWR);
	gp2x_ram=(Uint8 *)mmap(0, 0x1000000, PROT_READ|PROT_WRITE, MAP_SHARED, 
				gp2x_dev_mem, 0x02000000);
	//printf("gp2x_ram %p\n",gp2x_ram);
	gp2x_memregl=(Uint32 *)mmap(0, 0x10000, PROT_READ|PROT_WRITE, MAP_SHARED, 
				    gp2x_dev_mem, 0xc0000000);
	gp2x_memregs=(Uint16*)gp2x_memregl ;
}

Uint8 *gp2x_ram_malloc(size_t size) {
	static volatile Uint8 *ram_ptr=0;
	volatile Uint8 *t;
	if (!ram_ptr) {
		ram_ptr=gp2x_ram/*+0x8000+0x100000*/;
		//printf("Ram_ptr=%p\n",ram_ptr);
	}
	if ((Uint32)ram_ptr-(Uint32)gp2x_ram+size<=0x1000000) {
		t=ram_ptr;
		ram_ptr+=(((Uint32)size)|0xF)+0x1;
		//printf("allocating %d\n",size);
		return t;
	} else {
		printf("Out of memory\n");
	}
	return NULL;
}

void gp2x_sound_volume(int l, int r) {
	static int L;
	L=(((l*0x50)/100)<<8)|((r*0x50)/100);          //0x5A, 0x60
	ioctl(gp2x_mixer, SOUND_MIXER_WRITE_PCM, &L); //SOUND_MIXER_WRITE_VOLUME
}

void gp2x_init(void) {
	int overclock=CF_VAL(cf_get_item_by_name("cpu_speed"));
	char clockgen = 0;
        unsigned sysfreq=0;

	gp2x_ram_init();
	if (CF_BOOL(cf_get_item_by_name("sound"))) {
		gp2x_mixer=open("/dev/mixer", O_RDWR);
		if (gp2x_mixer==-1) {
			printf("Can't open mixer\n");
		}
	}
	/* save FCLOCK */
	sysfreq=get_freq_920_CLK();
        sysfreq*=get_920_Div()+1;
        cpufreq=sysfreq/1000000;
	if (overclock!=0) {
		if (overclock<66) overclock=66;
		if (overclock>320) overclock=320;
		set_FCLK(overclock);
	}

	gn_init_skin();
}

void gp2x_quit(void) {

	if (gp2x_dev_mem!=-1) close(gp2x_dev_mem);
	if (gp2x_gfx_dump!=-1) close(gp2x_gfx_dump);
	if (gp2x_mixer!=-1) close(gp2x_mixer);

	if (CF_VAL(cf_get_item_by_name("cpu_speed"))!=0) {
		set_FCLK(cpufreq);
	}

	sync();
	SDL_Quit();
	
	if (strcmp("null",CF_STR(cf_get_item_by_name("frontend")))!=0) {
		execl(CF_STR(cf_get_item_by_name("frontend")),
		      CF_STR(cf_get_item_by_name("frontend")),NULL);
	} else {
		/*
		  chdir("/usr/gp2x");
		  execl("gp2xmenu","gp2xmenu",NULL);
		*/
	}


}

#endif
